%ftmp_model_final.
% program for "A fiscal theory of monetary policy with partially-repaid
% long-term debt" 
% Solves ftmp model with price stickiness and long term debt
% Final version. Earlier versions document all sorts of variations and robustness checks.  
% self contained with funcitons at the end

clear all 
close all  

newb = 1; % search for bi to set sum omega^j pi_j to 0. 

% model parameters
% Note the notation in this program reflects an earlier draft
% of the paper.

bet = 0.99; % 
omeg = 0.9; % 0.7 in draft, but larger illustrates long term debt effects better. maturity coeff
alph = 0.2; % 
sig = 0.5; %
kap = 0.5; %
%kap = 100000; %to produce flex price row of table. Over writes graphs!
rhoi = 0.7; %
rhos = 0.5; 
rho = 0.99; %rho = 1  works too
t_ix = 0.5; % these are set to zero below in the case of no rules 
t_ipi = 0.8;
t_sx = 1;
t_spi = 0.25;
b_i = 0;
b_s= 0; % initialize so functions have an argument. Should not be used before defined. 

fraction_inflated = 0.4; % ratio of sum omega^j pi_j to sum rho^j u_j in fiscal shock. determines b_s

% control parameters
H = 100; % horizon of impulse response function 

%%%%%%%%%%%%%%%%%
%MP shock, no rules
%%%%%%%%%%%%%%%%%

shock = [0.01;0];
b_s = 0; % just to be sure, checked that it is not used. 

disp('mp shock, no rules')

if newb; % search to find bi
    b_i_guess=[0 1]';
    f = @(b_i)parameterfun(sig,kap,bet,omeg,rho,0*t_ix,0*t_ipi,rhoi,rhos,b_i,b_s,H,t_spi,t_sx,alph,shock);
    [b_i,sumomeg] = fsolve(f,b_i_guess);
    b_i = mean(b_i);
end;

% now re-solve with that b_i to produce useable results in workspace

[N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,0*t_ix,0*t_ipi,rhoi,rhos,b_i,b_s);

[zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio] =...
f_doir_final(H,Nb,nb,N,Q,ze,Lb,0*t_ipi,0*t_ix,0*t_spi,0*t_sx,alph,omeg,b_s,b_i,shock,rho);

displayresults(b_s,b_i,rho,st, pit, rnt, ust, omeg, sumomeg, H);

% MP picture, no rules

figure
colors = get(gca,'colororder');
hold on

plot((0:H-1)',100*[it],'linewidth',2)
text(0.4,0.5,'i, u_i','color',colors(1,:),'fontsize',16,'backgroundcolor','w')

text(1.1,100*pit(2),'\pi','color',colors(2,:),'fontsize',16,'backgroundcolor','w')
plot((0:H-1)',100*[pit],'-v','linewidth',2)

text(1.1,100*xt(2),'x','color',colors(3,:),'fontsize',16,'backgroundcolor','w')
plot((0:H-1)',100*[xt],'-o','linewidth',2)

text(1,100*st(2)+0.15,'s','color',colors(4,:),'fontsize',16)
plot((0:H-1)',100*[st],'-s','linewidth',2)

text(0.95,100*vt(2)+0.25,'v','color',colors(5,:),'fontsize',16,'backgroundcolor','w')
plot((0:H-1)',100*[vt ],'-','linewidth',2)

plot((0:H-1)',100*rnt,'--','linewidth',2, 'color', colors(7,:))
text(1.2,100*rnt(2)+.2,'r^n','color',colors(7,:),'fontsize',16)

plot((0:H-1)', 0*(0:H-1)','-k');
%legend('i','\pi','x','s','v','v^*');

axis([0 7 -inf inf])
%title('Response to a monetary policy shock')
ylabel('Percent')
xlabel('Time')
print -depsc mp_response_simple_new.eps


%%%%%%%%%%%
% MP, case with policy rule responses
%%%%%%%%%%

shock = [0.01;0];

disp('-----------');
disp('mp shock, yes rules')

if newb; % search to find bi
    b_i_guess=[0 1]';
    f = @(b_i)parameterfun(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s,H,t_spi,t_sx,alph,shock);
    [b_i,sumomeg] = fsolve(f,b_i_guess);
    b_i = mean(b_i);
end;

% now re-solve with that b_i to produce useable results in workspace

[N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s);

[zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio] =...
f_doir_final(H,Nb,nb,N,Q,ze,Lb,t_ipi,t_ix,t_spi,t_sx,alph,omeg,b_s,b_i,shock,rho);

displayresults(b_s,b_i,rho,st, pit, rnt, ust, omeg, sumomeg, H);


figure
hold on
colors = get(gca,'colororder');

plot((0:H-1)',100*[it],'linewidth',2)
text(1,100*it(2),'i','color',colors(1,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)',100*[pit],'-v','linewidth',2)
text(1.95,100*pit(3),'\pi','color',colors(2,:),'fontsize',16,'backgroundcolor','w')
% text(8,100*pit(9),'\pi','color',colors(3,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)',100*[xt],'-o','linewidth',2)
text(2.95,100*xt(4),'x','color',colors(3,:),'fontsize',16,'backgroundcolor','w')

text(0.95,100*st(2)-0.1,'s','color',colors(4,:),'fontsize',16,'backgroundcolor','w')
plot((0:H-1)',100*[st],'-s','linewidth',2)

plot((0:H-1)',100*[vt],'-','linewidth',2)
text(4,100*vt(5)-0.15,'v','color',colors(5,:),'fontsize',16)

plot((0:H-1)',100*[uit],'--','linewidth',2)
text(1.2,100*uit(2)-0.1,'u^i','color',colors(6,:),'fontsize',16,'backgroundcolor','w')

text(1.2,100*rnt(2)+0.2,'r^n','color',colors(7,:),'fontsize',16,'backgroundcolor','w')
plot((0:H-1)',100*rnt,'--','linewidth',2)

plot((0:H-1)', 0*(0:H-1)','-k');

axis([0 7 -inf inf])
%title('Response to a monetary policy shock')
ylabel('Percent')
xlabel('Time')
print -depsc mp_response_thetas_new.eps

%%%%%%%%%%%%%%%%%%%%%%
% fiscal policy response, no rules
%%%%%%%%%%%%%%%%%%%%%%

shock = [0;-0.01];
disp('-------------'); 
disp('fp shock, no rules')
b_i = 0; % jsut to be sure, I checked though that not used. 

if newb; % search to find bs
    b_s_guess=[0 1]';
    f = @(b_s)parameterfun_s(sig,kap,bet,omeg,rho,0*t_ix,0*t_ipi,rhoi,rhos,b_i,b_s,H,0*t_spi,0*t_sx,alph,shock,fraction_inflated);
    [b_s,sumratio] = fsolve(f,b_s_guess);
    b_s = mean(b_s);
end;

% now re-solve with that b_s to produce useable results in workspace

[N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,0*t_ix,0*t_ipi,rhoi,rhos,b_i,b_s);

[zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio] =...
f_doir_final(H,Nb,nb,N,Q,ze,Lb,0*t_ipi,0*t_ix,0*t_spi,0*t_sx,alph,omeg,b_s,b_i,shock,rho);

displayresults(b_s,b_i,rho,st, pit, rnt, ust, omeg, sumomeg, H);

% fp picture

figure
colors = get(gca,'colororder');
plot((0:H-1)',100*[it],'linewidth',2)
hold on;
plot((0:H-1)',100*[pit],'-v','linewidth',2)
plot((0:H-1)',100*[xt],'-o','linewidth',2)

plot((0:H-1)',100*[st],'-s','linewidth',2)
plot((0:H-1)',100*[vt ],'-','linewidth',2)
%    plot((0:H-1)',100*[vstart ],'--*','linewidth',2)

plot((0:H-1)',100*[ust],'--','linewidth',2)

%legend('i','yield','\pi','s','v','v^*','u^s');
colors = get(gca,'colororder');
text(1,100*it(2),'i, r^n','color',colors(1,:),'fontsize',16,'backgroundcolor','w')
text(1,100*pit(2)+.07,'\pi','color',colors(2,:),'fontsize',16,'backgroundcolor','w')
text(1,100*xt(2)-.07,'x','color',colors(3,:),'fontsize',16)
text(1.9,100*st(3),'s','color',colors(4,:),'fontsize',16,'backgroundcolor','w')
text(3,100*vt(4)-0.1,'v','color',colors(5,:),'fontsize',16)
text(3,100*ust(4),'u^s','color',colors(6,:),'fontsize',16,'backgroundcolor','w')

axis([0 7 -1 inf])

%title('Model response to fiscal shock')
ylabel('Percent')
xlabel('Time')
print -depsc fp_response_simple_new.eps

%%%%%%%%%%%%%%%%
% fp, rules
%%%%%%%%%%%%%%%%

shock = [0;-0.01];
disp('-------------'); 
disp('fp shock, with rules')

if newb; % search to find bs
    b_s_guess=[0 1]';
    f = @(b_s)parameterfun_s(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s,H,t_spi,t_sx,alph,shock,fraction_inflated);
    [b_s,sumratio] = fsolve(f,b_s_guess);
    b_s = mean(b_s);
end;

% now re-solve with that b_s to produce useable results in workspace

[N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s);

[zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio] =...
f_doir_final(H,Nb,nb,N,Q,ze,Lb,t_ipi,t_ix,t_spi,t_sx,alph,omeg,b_s,b_i,shock,rho);

displayresults(b_s,b_i,rho,st, pit, rnt, ust, omeg, sumomeg, H);

% fp picture
figure
colors = get(gca,'colororder');
plot((0:H-1)',100*[it],'linewidth',2)
text(1.5,100*it(2)-0.05,'i','color',colors(1,:),'fontsize',16)

hold on

plot((0:H-1)',100*[pit],'-v','linewidth',2)
text(1.5,100*pit(2)+0.04,'\pi','color',colors(2,:),'fontsize',16)

plot((0:H-1)',100*[xt],'-o','linewidth',2)
text(2.2,100*xt(3),'x','color',colors(3,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)',100*[st],'-s','linewidth',2)
text(2,100*st(3),'s','color',colors(4,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)',100*[vt],'-','linewidth',2)
text(4,100*vt(5)+0.05,'v','color',colors(5,:),'fontsize',16)

plot((0:H-1)',100*[ust],'--','linewidth',2)
text(3,100*ust(4),'u^s','color',colors(6,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)',100*rnt,'--','linewidth',2)
text(1,100*rnt(2),'r^n','color',colors(7,:),'fontsize',16,'backgroundcolor','w')

plot((0:H-1)', 0*(0:H-1)','-k');


axis([0 7 -inf inf])

%title('Model response to fiscal shock')
ylabel('Percent')
xlabel('Time')
print -depsc fp_response_thetas_new.eps

%%%%%%%%%%%%%%%
% simple example No longer in paper but preserved for other uses.
%%%%%%%%%%%%

if 0;
    % [s v u]_t+1' = A [s v u]_t + B eps_t+1
    alph_1 = 0.1;
    rhos_1 = 0.7;
    b_s_1 = 1;
    yt=simple_model(alph_1,rhos_1,rho,b_s_1,H);
    yt2=simple_model(alph_1,rhos_1,rho,0,H);
    
    fig_1 = figure;
    colors = get(fig_1,'defaultaxescolororder');
    plot((0:H-1)',yt, 'linewidth',2);
    hold on,
    plot((0:H-1)',yt2(:,1:2),'--','linewidth',2)
    plot((0:H-1)',0*(0:H-1)','-k')
    axis([0 25 -1.1 2.2]);
    
    text(8,0.3,'s','color',colors(1,:),'fontsize',18);
    text(12,1.5,'v','color',colors(2,:),'fontsize',18);
    text(7,-0.2,'u','color',colors(3,:),'fontsize',18);
    xlabel('Time','fontsize',18)
    print -depsc2 simple_example_ir.eps
    
end;

%%%%%%%%%%%%%%%%%%
%Function definitions
%%%%%%%%%%%%%%%%%%

function [zt,yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio]=...
    f_doir_final(H,Nb,nb,N,Q,ze,Lb,t_ipi,t_ix,t_spi,t_sx,alph,omeg,b_s,b_i,shock,rho);

zbt = zeros(H,Nb);
zbt(2,:) = (ze*shock)';
for t = 3:H;
    zbt(t,:) = Lb*zbt(t-1,:)';
end;

% now create the full z with zeros
zt = zeros(H,N);
zt(:,nb) = zbt;

% create original variables
yt = zt*Q';

xt = yt(:,1);
pit = yt(:,2);
qt = yt(:,3);
uit = yt(:,4);
ust = yt(:,5);

rnt = zeros(H,1);
rnt = omeg*qt-[0;qt(1:H-1)];
vt = zeros(H,1);
for t = 2:H;
    vt(t) = 1/rho*((1-alph)*vt(t-1) + rnt(t) - (1+t_spi)*pit(t) - t_sx*xt(t) - ust(t));
end;

it = t_ipi*pit + t_ix*xt + uit;
st = t_spi*pit + t_sx*xt + alph*[0;vt(1:end-1)] + ust;
%yield is nonlinear so use the right size
qlevelt = qt - log(1-omeg);
yldt = exp(-qlevelt)+omeg-1; 

sumomeg = (sum((omeg.^(0:1:H-2)').*pit(2:H))*100);
rhoj = [0; rho.^(0:1:size(st,1)-2)'];
sumratio = (-sum((omeg.^(0:1:H-2)').*pit(2:H))/sum(rhoj.*ust));

end

function displayresults(b_s,b_i,rho,st, pit, rnt, ust, omeg, sumomeg, H); 
disp('----------------');
disp(' bs bi ');
disp([ b_s b_i ]);

rhoj = [0; rho.^(0:1:size(st,1)-2)'];

disp('terms of inflation decomposition')
disp('unexpected pi, rn, sum s, sum rn - pi')
disp(100*[pit(2) rnt(2) sum(rhoj.*st) sum(rhoj(3:end).*(rnt(3:end)-pit(3:end)))]);
%disp(100*[pit(2) (rnt(2)-sum(st)+sum(rnt(3:end)-pit(3:end)))]); % chec

disp('unexpected sum  omega j pij, sum s, sum (1-omega^j) (rn - pi)')
disp(100*[sum((omeg.^(0:1:H-2)').*pit(2:H)) sum(rhoj.*st) ...
    sum((rhoj(3:end)-omeg.^(1:1:H-2)').*(rnt(3:end)-pit(3:end)))]);

disp('unexpected bond return decomposition')
disp('rn1  sum_rn sum_pi sum_rn-pi');
disp(100*[rnt(2) ...
    sum((omeg.^(1:1:H-2)').*rnt(3:end)) ...
    sum((omeg.^(1:1:H-2)').*pit(3:end))  ...
    sum((omeg.^(1:1:H-2)').*(rnt(3:end)-pit(3:end)))]);

disp('sum of us, sum of s, s<0 and s>0');
rhost = rhoj.*st; 
disp(100*[ sum(rhoj.*ust) sum(rhoj.*st) sum(rhost(rhost<0)) sum(rhost(rhost>0))])

if sum(rhoj.*ust)>0
    disp('-sum of omega^j pij / sum us');
    disp(-sum((omeg.^(0:1:H-2)').*pit(2:H))/sum(rhoj.*ust));
end

end


function sumomeg = parameterfun(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s,H,t_spi,t_sx,alph,shock);
    sumomeg = 0*b_i;
    for ind = 1:size(b_i,1);
        [N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i(ind),b_s);
        [zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg(ind),sumratio] =...
           f_doir_final(H,Nb,nb,N,Q,ze,Lb,t_ipi,t_ix,t_spi,t_sx,alph,omeg,b_s,b_i(ind),shock,rho);
    end
end


function sumratio = parameterfun_s(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s,H,t_spi,t_sx,alph,shock,fraction_inflated);
    sumratio = 0*b_s;
    for ind = 1:size(b_s,1);
        [N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s(ind));
        [zt, yt, xt, pit, vt, qt, uit, ust, it, st, qlevelt, yldt, rnt,sumomeg,sumratio(ind)] =...
           f_doir_final(H,Nb,nb,N,Q,ze,Lb,t_ipi,t_ix,t_spi,t_sx,alph,omeg,b_s(ind),b_i,shock,rho);
       sumratio(ind) = sumratio(ind)-fraction_inflated;
    end
end


function yt=simple_model(alph,rhos,rho,b_s,H);
A = [ 0 alph rhos ;
    0 rho^(-1)*(1-alph) -rho^(-1)*rhos ;
    0   0                 rhos];
B = -[ 1;
    rho^(-1)*(b_s-1);
    1];

yt = zeros(H,3);
yt(2,:) = B';
for t = 3:H;
    yt(t,:) = A*yt(t-1,:)';
end;
end


function [N,Nb,nb,Q,ze,Lb] = solvemodel(sig,kap,bet,omeg,rho,t_ix,t_ipi,rhoi,rhos,b_i,b_s)
% matrices
show_results = 0; % use for debugging
A = eye(5);
N = size(A,1);

% x pi q ui us

B = [...
    (1+sig*t_ix+sig*kap/bet) sig*t_ipi-sig/bet    0       sig    0 ;
    -kap/bet                 1/bet                0        0     0 ;
    t_ix/omeg                t_ipi/omeg           1/omeg 1/omeg  0 ;
    0                        0                    0       rhoi   0 ;
    0                        0                    0        0    rhos];



C = [...
    0 0 ;
    %b_i b_s ;
    -b_i -b_s;
    0 0 ;
    1 0 ;
    0 1 ];

D = [...
    1  0 ;
    0 0;
    0  1 ;
    0  0 ;
    0  0 ];
 
% Solve by eigenvalues

A1 = inv(A); 
F = A1*B; 
[Q L] = eig(F); 
Q1 = inv(Q); 
if show_results;
    disp('Eigenvalues');
    disp(abs(diag(L)'));
end
% produce Ef, Eb, that select forward and backward
% eigvenvalues. If L>=1 in position 1, 3, 
% produce
% 1 0 0 0 
% 0 0 1 0 ...
% for example

nf = find(abs(diag(L))>=1); % nf is the index of eigenvalues greater than one
if show_results
    disp('number of eigenvalues >=1');
    disp(size(nf,1))
end
if (size(nf,1) < size(D,2)); 
    disp('not enough eigenvalues greater than 1');
end;
Ef = zeros(size(nf,1),size(A,2));
Efstar = zeros(size(A,2),size(A,2));
for indx = 1:size(nf,1);
    Ef(indx,nf(indx))=1; 
    Efstar(nf(indx),nf(indx)) = 1; 
end;

nb = find(abs(diag(L))<1);
Eb = zeros(size(nb,1),size(A,2));
for indx = 1:size(nb,1);
    Eb(indx,nb(indx))=1;
end;

ze = Eb*Q1*A1*(C-D*inv(Ef*Q1*A1*D)*Ef*Q1*A1*C);
% how epsilon shocks map to z.
% in principle the forward z are zero. In practice they are 1E-16 and then
% grow. So I go through the trouble of simulating forward only the nonzero
% z and eigenvalues less than one.

Nb = size(Eb,1); % number of stable z's
Lb = (Eb*L*Eb'); % diagonal with only stable Ls
end
