% get_data_and_run_VAR
% script common to inflation and value decomposition. 

% load Hall and Sargent data.

x = load('data/US_holding_period_return_1790_2018.csv');
x = x(x(:,1)>=1929,:);
yr_r = x(:,1);
mo_r = x(:,2);
ret = x(:,3);

x = load('data/US_Treasury_Debt_1776-2018_edited.csv');
x = x(((x(:,1)==1928)&(x(:,2)==12))|(x(:,1)>=1929),:); % start with 1928:12 for baseline
yr = x((x(:,1)>=1929),1);
mo = x((x(:,1)>=1929),2);
gmv = x(:,3); %Market Value held by public,
omv = x(:,3); % to allow rest of program to work. The distinction is not maintained with new data
clear x

% create surplus, make all series the same length

gps = -gmv(2:end)+gmv(1:end-1).*(1+ret); % nominal surplus, gross debt
ops = -omv(2:end)+omv(1:end-1).*(1+ret); % nominal surplus, outstanding debt


% plots to diagnose 2011  surpluses
if 0
    figure
    plot(datenum(yr,mo,28),[(gmv(2:end)-gmv(1:end-1))./gmv(1:end-1) ret ])
    datetick('x')
    %
    figure
    plot(datenum(yr,mo,28),filter(ones(12,1)/12,1,gps))
    hold on
    plot(datenum(yr,mo,28),filter(ones(12,1)/12,1,dgps))
    datetick('x')
    dgps = -gmv(2:end)+gmv(1:end-1);
    figure
    plot(datenum(yr,mo,28),filter(ones(12,1)/12,1,gps))
    hold on
    plot(datenum(yr,mo,28),filter(ones(12,1)/12,1,dgps))
end


gmv28 = gmv(1);
omv28 = omv(1); % save initial values for later
gmv = gmv(2:end); % eilminate 1928:12
omv = omv(2:end);

% create annual, quarterly data

Tm = size(gps,1);
ta = 1;
for t = (12:12:Tm);
    yr_a(ta) = yr(t); % year is 12/31 of that year
    mo_a(ta) = 12;
    gmv_a(ta) = gmv(t); % use end of year value
    omv_a(ta) = omv(t);
    gps_a_sum(ta) = sum(gps(t-11:t)); % sum for comparison
    ops_a_sum(ta) = sum(ops(t-11:t));
    gps_a(ta) = gps(t-11);
    ops_a(ta) = ops(t-11);
    for indx = -10:1:0; % reinvest surpluses so the identity works annual data
        gps_a(ta) = gps_a(ta)*(1+ret(t+indx)) + gps(t+indx);
        ops_a(ta) = ops_a(ta)*(1+ret(t+indx)) + ops(t+indx);
    end;
    % to check that identity works.
    if t == 12;
        gret_a(ta) = (gmv_a(ta)+gps_a(ta))/gmv28-1; %net return that makes identity hold
        oret_a(ta) = (omv_a(ta)+ops_a(ta))/omv28-1;
    else;
        gret_a(ta) = (gmv_a(ta)+gps_a(ta))/gmv_a(ta-1)-1; %net return that makes identity hold
        oret_a(ta) = (omv_a(ta)+ops_a(ta))/omv_a(ta-1)-1;
    end;
    cumret_a(ta) = prod(1+ret(t-11:t))-1; % direct cumulative return for comparison
    ta = ta+1;
end
yr_a = yr_a';
mo_a = mo_a';
gps_a_sum = gps_a_sum';
ops_a_sum = ops_a_sum';
gps_a = gps_a';
ops_a = ops_a';
gmv_a = gmv_a';
omv_a = omv_a';
gret_a = gret_a';
oret_a = oret_a';
cumret_a = cumret_a';

% diagnose 2011 -- not a problem of annualizing
if 0
    figure
    plot(datenum(yr,mo,28),filter(ones(12,1),1,gps))
    hold on
    plot(datenum(yr_a,mo_a,28),gps_a);
    plot(datenum(yr_a,mo_a,28),gps_a_sum);
end

% quarterly data. Not currently used. 
tq = 1;
for t = (3:3:Tm);
    yr_q(tq) = yr(t); % date is last mo of quarter
    mo_q(tq) = mo(t);
    omv_q(tq) = omv(t);
    ops_q_sum(tq) = sum(ops(t-2:t)); % sum for comparison
    ops_q(tq) = ops(t-2);
    for indx = -1:1:0; % reinvest surpluses
        ops_q(tq) = ops_q(tq)*(1+ret(t+indx)) + ops(t+indx);
    end;
    % to check that identity works.
    if t == 3;
        oret_q(tq) = (omv_q(tq)+ops_q(tq))/omv28-1;  %net return that makes identity hold
    else;
        oret_q(tq) = (omv_q(tq)+ops_q(tq))/omv_q(tq-1)-1; %net return that makes identity hold
    end;
    cumret_q(tq) = prod(1+ret(t-2:t))-1; % direct cumulative return for comparison
    tq = tq+1;
end

ops_q_sum = ops_q_sum(yr_q>=1947)';
ops_q = ops_q(yr_q>=1947)';
omv_q = omv_q(yr_q>=1947)';
oret_q = oret_q(yr_q>=1947)';
cumret_q = cumret_q(yr_q>=1947)';
mo_q = mo_q(yr_q>=1947)';
yr_q = yr_q(yr_q>=1947)';

%Load GDP other data, create ratios

x = load('data/GDPA.csv');
x = x(x(:,1)<=2018,:);
yr_GDP = x(:,1);
GDP = x(:,4);

x = load('data/A191RD3A086NBEA.csv');
x = x(x(:,1)<=2018,:);
GDPDEF = x(:,4)/100;
clear x

RGDP = GDP./GDPDEF; % to keep identities consistent;

x = load('data/pcecca.csv');
x = x(x(:,1)<=2018,:);
rcons = x(:,4); % real and nominal consumption. Use instead of GDP to detrend

x = load('data/pceca.csv');% 1929 on
x = x(x(:,1)<=2018,:);
ncons = x(:,4);

% take ratios

if use_consumption
    
    py_a = ncons*1e9*mean(GDP./ncons); % adjust up so units are x to GDP ratios
    Y_a = py_a./GDPDEF; % inflaiton is GDP, so keep consistent.
    
else
    disp('get_data_and_run_var: using GDP. This is nonstandard.')
    py_a = GDP*1e9;
    Y_a = py_a./GDPDEF;
end

ovy_a = omv_a./py_a;
gvy_a = gmv_a./py_a;
osy_a = ops_a./py_a;
gsy_a = gps_a./py_a;


% nipa data for comparison -- from simple_debt_graph

x = load('data/FYGFDPUB.csv');%1939-2016
% gross federal debt held by public
x = x((x(:,1)>=1929)&(x(:,1)<=2018),:);
B_yr = x(:,1);
B = x(:,4);

x = load('data/usrec.csv');
% NBER recressions
rec_yr = x(:,1);
rec_mo = x(:,2);
rec_dy = x(:,3);
rec = x(:,4);

%unemployment
x = load('data/unrate.csv'); % monthly from 194801
unm_yr = x(:,1);
unm_mo = x(:,2);
unm_dy = x(:,3);
unm = x(:,4);

x = load('data/M0892AUSM156SNBR.csv'); % early monthly data 29:04 to 42:06
unem_yr = x(:,1);
unem_mo = x(:,2);
unem_dy = x(:,3);
unem = x(:,4);

x = load('data/Q0892BUSQ156SNBR.csv'); % early quarterly data 40:04 to 46:10
uneq_yr = x(:,1);
uneq_mo = x(:,2);
une1_dy = x(:,3);
uneq = x(:,4);

% splice together to form annual unemployment conformable to yr_a

un = 0*yr_a;
un(1) = mean(unem(1:9)); %1929
for ta = 2:13; %1930:1941
    un(ta) = mean(unem((ta-2)*12+10:(ta-2)*12+21));
end;
for ta = 14:18; %1942:1946
    un(ta) = mean(uneq((ta-14)*4+8:(ta-14)*4+11));
end;
un(19) = NaN; % 1947
for ta=20:size(yr_a,1); % 1948-2018
    un(ta) = mean(unm((ta-20)*12+1:(ta-20)*12+12));
end;


x = load('data/fyfsd.csv');%1901-2017
% ferderal surplus or deficit, US OMB
x = x((x(:,1)>=1929)&(x(:,1)<=2018),:);
sg_yr = x(:,1);
sg = x(:,4);

x = load('data/A091RC1Q027SBEA.csv');%1947-2018
% federal government current expenditures, interest payments, BEA
x = x((x(:,1)>=1929)&(x(:,1)<=2018),:);
int_cost_yr = x(:,1);
int_cost = x(:,4);
% convert to annual
int_cost_yr_a = zeros(2018-1947+1,1);
int_cost_a = zeros(2018-1947+1,1);
for ta = 1947:2018;
    int_cost_yr_a(ta-1947+1) = ta;
    int_cost_a(ta-1947+1) = mean(int_cost(4*(ta-1947)+(1:4)));
end;

% form ratios in nipa data. Divide by consumpiton
% like our data

if use_consumption;
    disp('using ratios to consumption')
    sg_y = (sg/1000)./ncons/mean(GDP./ncons);
    s_y = (sg(find(sg_yr==1947):end)/1000 + int_cost_a)./...
        ncons(find(yr_a==1947):end)/mean(GDP./ncons);
    B_y = B./ncons(find(yr_a>=1939):end)/mean(GDP./ncons);
else; %  GDP for comparison
    disp('using ratios to GDP')
    sg_y = (sg/1000)./GDP;
    s_y = (sg(find(sg_yr==1947):end)/1000 + int_cost_a)./...
        GDP(find(yr_GDP==1947):end);
    B_y = B./GDP(find(yr_GDP==1939):end);
end


% quarterly data

x = load('data/gdp.csv');
gdp_q = x(:,4);
gdp_q = gdp_q(x(:,1)<=2018);

x = load('data/ngdppot.csv'); % only 1949 on
gdppot_q = x(:,4);
gdppot_q = gdppot_q(x(:,1)<=2018);

x = load('data/gdpdef.csv');
gdpdef_q = x(:,4);
gdpdef_q = gdpdef_q(x(:,1)<=2018);

rgdp_q = gdp_q./gdpdef_q;

ovy_q = omv_q./(gdp_q*1e9);
osy_q = ops_q./(gdp_q*1e9);


% interest rate data

x=load('data/crsp_riskfree_1_mo.txt');
caldt = x(:,2:4);
y1m = x(:,5);

x=load('data/crsp_riskfree_3_mo.txt');
caldt = x(:,2:4);
y = x(:,5);

y1m_a = log(1+y1m(((caldt(:,1)==12)&(caldt(:,3)<=2018)&(caldt(:,3)>=1929)),:)/100);
y_a = log(1+y(((caldt(:,1)==12)&(caldt(:,3)<=2018)&(caldt(:,3)>=1929)),:)/100);

% use last day of year for annual data

indx1 = find((caldt(:,1)==3)&(caldt(:,3)==1947));
indx2 = find((caldt(:,1)==12)&(caldt(:,3)==2018));

y1m_q = log(1+y1m(indx1:3:indx2,:)/100);
y_q = log(1+y(indx1:3:indx2,:)/100);

% long term bond data

x = load('data/GS10.csv');

y10_yr = x(:,1);
y10_mo = x(:,2);
y10_dy = x(:,3);
y10 = x(:,4);

x = load('data/ltgov.csv'); % used to extend back before 1950s
lg_yr = x(:,1);
lg_mo = x(:,2);
lg_dy = x(:,3);
lg = x(:,4);

indx_last = find((y10_yr(1)==lg_yr)&(lg_mo==y10_mo(1))&(lg_dy==y10_dy(1)));

y10_yr = [lg_yr(1:indx_last-1); y10_yr];
y10_mo = [lg_mo(1:indx_last-1); y10_mo];
y10_dy = [lg_dy(1:indx_last-1); y10_dy];
y10 = [lg(1:indx_last-1); y10];

indx1 = find( (y10_yr == 1941) & (y10_mo == 12)); % first useable date is 1942 jan 1 = 1941 year data
indx2 = find( (y10_yr == 2018) & (y10_mo == 12));

% y10_a = log(1+y10(indx1+1:12:indx2+1)/100);
% use Jan 1 year 1953 for 1952 observation since they are first of the month
y10_a = log(1+y10(indx1:12:indx2)/100);
y10_yr_a = y10_yr(indx1:12:indx2);

% this is now 1942 to 2018

% use historic Fed  data to fill in 1929 to  1940

x = load('data/FRB_H15.csv');
y10p_yr_m  = x(:,1);
y10p_mo_m = x(:,2);
y10p_m = x(:,3);
%use dec observation for the year

y10p_yr = y10p_yr_m(60:12:end);
y10p_mo = y10p_mo_m(60:12:end);
y10p = y10p_m(60:12:end);

y10p_yr = y10p_yr(1:12); % stop in 1940
y10p = y10p(1:12);


y10_a = [log(1+y10p/100) ; y10_a];
y10_yr_a = [y10p_yr; y10_yr_a];


if 0;
    figure;
    plot(datenum(caldt(:,[3 1 2])),[y1m y]);
    legend('1m','3m');
    % shows it's better to use 3 m
end;

% ---------
% linearized analysis
% form logs for analysis of linearized variables.
% ----------------

vt = log(ovy_a); % choose o or g
rnt = log(1+cumret_a);
inflt = [NaN; log(GDPDEF(2:end)./GDPDEF(1:end-1))];
grt = [NaN; log(Y_a(2:end)./Y_a(1:end-1))];
syt = osy_a;
svt = [NaN; osy_a(2:end)./ovy_a(1:end-1)];
svt_x = [ NaN; -vt(2:end)+vt(1:end-1)+rnt(2:end)-inflt(2:end)-grt(2:end)]; % use to check identities.

vt_q = log(ovy_q);
rnt_q = log(1+cumret_q);
inflt_q = [NaN; log(gdpdef_q(2:end)./gdpdef_q(1:end-1))];
grt_q = [NaN; log(rgdp_q(2:end)./rgdp_q(1:end-1))];
syt_q = osy_q;
svt_q = [NaN; osy_q(2:end)./ovy_q(1:end-1)];
svt_qx = [ NaN; -vt_q(2:end)+vt_q(1:end-1)+rnt_q(2:end)-inflt_q(2:end)-grt_q(2:end)]; % use to check identities.


%Plot data

% comparison with BEA
% moved to make_surplus_plot

if 0 % Important: if you get in mind to use quarterly data, deal with the huge seasonal shown here
    figure
    plot(eomdate(yr_q,mo_q),[ svt_q svt_qx]);
    title('Quarterly: actual sv and sv calcualted to nail identity')
    datetick('x');
end


% ------------------
% Run VAR
% -----------------

stind = find(yr_a == stdate);
stind_y10 = find(y10_yr_a == stdate);
endind = find(yr_a == enddate);
endind_y_10 = find(y10_yr_a == enddate);
disp('Using sample' )
disp([stdate enddate])

% set up data
x = [rnt(stind:endind) grt(stind:endind) inflt(stind:endind) svt_x(stind:endind) vt(stind:endind) y_a(stind:endind) y10_a(stind_y10:endind)];

lbls = {'r^n','g', '\pi','s','v','i','y'};
rni = 1; % index numbers, used later to select elments
gi = 2;
pii = 3;
svi = 4;
vi = 5;
yi = 6;
y10i = 7;
rgi = 8;
rngi = 9;

arn  = [ 1 0 0 0 0 0 0 ]'; % a'x gives elements
ag   = [ 0 1 0 0 0 0 0 ]';
api  = [ 0 0 1 0 0 0 0 ]';
asv  = [ 0 0 0 1 0 0 0 ]';
av   = [ 0 0 0 0 1 0 0 ]';
ay3  = [ 0 0 0 0 0 1 0 ]';
ay10 = [ 0 0 0 0 0 0 1 ]';

% estimate VAR
% IF you change following, mirror in bootstrap or place in function

xl  = x; %  save full sample for later use

x1 = x(2:end,:);
x = [ones(size(x,1)-1,1) x(1:end-1,:)];

b = x\x1; % Estimate by OLS
err = x1-x*b;
sigma = cov(err);
R2 = 1-var(err)./var(x1);

A = b(2:end,:)';

[Q,V] = eig(A);
if any(any(abs(V)>=1));
    % this occurs in the pre 1980 sample. This is a hack to let the
    % program run -- produces an A matrix close to the estimated one If
    % you do pre 1980. pre 1980 data dropped. If you revive a pre-1980 
    % estimation, either defend this as bayesian, or report sums
    % of first 20 or so responses rather than I-A calculations.
    disp('Removing eigenvalues greater than one')
    Aold = A;
    V2 = (abs(V) < 1).*V + (abs(V)>=1)*0.99;
    A = real(Q*V2*inv(Q));
    disp('Old and new A matrix')
    disp(Aold)
    disp(A)
end;

ImA = inv(eye(size(A,1))-A);

% end estimation
