% inflation_decomposition 6. Main program to run inflation decompositions 
% for "The Fiscal Roots of Inflation." 
% It deletes many experiments and documentation of how they worked out
% keeping only final calcualtions. For those see inflation_decomposition_5
% it also may delete calculations used by other projects. 

clear all
close all
 
tex_files = 1; % print tables to files and overwrite files. Needs stdate = 1947 or 1930 or 1980
save_last_bs = 0; % don't wipe out last monte carlo when you do_bs = 1. Use to explore estimation variations while saving monte carlo
do_bs = 0; % do estimation, bootstrap/monte carlo? (If 0 loads last values. You need to use 1 the first time) 
Ntrial = 50000; % use 0 and save_last_bs = 1 to estimate wo monte carlo
stdate = 1947; %1930; %1947; %1980 choose sample. Growth rates start 1 period later. 1930 is min.  
enddate = 2018; % try 1980 to see first half. But not set up to save graphs and results for that period
use_consumption = 1; % switch to use GDP for simple graphs. Use consumption for VAR and real use. 


if do_bs; % redo bootstrap/monte carlo? (or load last one) Now also loads data and runs main VAR.
    %0 to play with plots and tables
    
    get_data_and_run_var; % common script with value_decomposition
    
    % -------------------------------------
    % calculate response to shocks
    % note: shocks should obey return identity!
    % ---------------------------------
    
    H = 50;
    N = size(x,2)-1;
    xt = zeros(H,N); % holds impulse response
    
    b2 = err(:,pii)\err; % regression of other errors on inflation shock
    xt(2,:) = b2; % unit inflation shock; orthog last
    for indx = 3:H; % find impulse response function
        xt(indx,:) = xt(indx-1,:)*A';
    end;
    
    % calculate decomposition terms
    
    [sumwpi,sum1wpi,sum1wrnpi,sum1mwrnpi,infl,rn,sums,sumg,sumrpi,omeg] = decode(find_decomp(xt,pii,rni,gi,svi));
    
   
    % note here and below. I use omeg that makes the inflation IR work.
    % if you reclaculate omeg you get small errors --
    % 0.98 instead of 1.00. A more evenhanded way to find omeg would be
    % better. function findomeg is an attempt, but didn't work. The effects
    % are small -- omeg is always 0.68-0.70 and changes in second decimal
    
    % AD shock
    
    b2g = [err(:,gi) err(:,pii) ]\err; % shock to gdp and inlation
    xtg(2,:) = -(b2g(1,:)+b2g(2,:))';
    for indx = 3:H; % find impulse response function
        xtg(indx,:) = xtg(indx-1,:)*A';
    end;
    [sumwpi_g,sum1wpi_g,sum1wrnpi_g,sum1mwrnpi_g,infl_g,rn_g,sums_g,sumg_g,sumrpi_g,omeg_g] =...
        decode(find_decomp(xtg,pii,rni,gi,svi));
    
    
    % long s+g shock
    
    b2s = [err*ImA'*(asv+ag) ]\err;
    longserr = [err*ImA'*(asv+ag) ]; % time series for later
    xts(2,:) = -b2s'; % deficit shock
    for indx = 3:H; % find impulse response function
        xts(indx,:) = xts(indx-1,:)*A';
    end;
    [sumwpi_s,sum1wpi_s,sum1wrnpi_s,sum1mwrnpi_s,infl_s,rn_s,sums_s,sumg_s,sumrpi_s,omeg_s] =...
        decode(find_decomp(xts,pii,rni,gi,svi));
   
     % long s+g no i shock
     
    b2si = [err*ImA'*(asv+ag) err(:,yi) err*A'*ay3 ]\err; % hold one period rate for two years
    xtsi(2,:) = -b2si(1,:)'; % deficit shock with no change in interest rate
    for indx = 3:H; % find impulse response function
        xtsi(indx,:) = xtsi(indx-1,:)*A';
    end;
    [sumwpi_si,sum1wpi_si,sum1wrnpi_si,sum1mwrnpi_si,infl_si,rn_si,sums_si,sumg_si,sumrpi_si,omeg_si] =...
        decode(find_decomp(xtsi,pii,rni,gi,svi));
  
    
    % dr shock
    
    % you need omeg to form ir, but don't know omeg till afterwareds.
    % hence iterate
    omeg_r_old = omeg;
    for iteration = 1:10;
        ImwAr = omeg_r_old*A*inv((eye(size(A,1))-omeg_r_old*A));
        ImAr = A*inv((eye(size(A,1))-A));
        b2r = [err*(ImAr'-ImwAr')*(arn-api)]\err;
        xtr(2,:) = -b2r'; % discount rate shock
        for indx = 3:H; % find impulse response function
            xtr(indx,:) = xtr(indx-1,:)*A';
        end;
        [sumwpi_r,sum1wpi_r,sum1wrnpi_r,sum1mwrnpi_r,infl_r,rn_r,sums_r,sumg_r,sumrpi_r,omeg_r] =...
            decode(find_decomp(xtr,pii,rni,gi,svi));
        disp([iteration omeg_r_old omeg_r sum1mwrnpi_r]);
        if abs(sum1mwrnpi_r - (-1.00)) < 0.0025;
            break
        else
            omeg_r_old = omeg_r;
        end
        if iteration >=10;
            disp('omega did not converge, raise iterations. new and old omega');
            disp([omeg_r_old omeg_r]);
        end
    end
    longrerr = err*(ImAr'-ImwAr')*(arn-api); % time series for later
    
    
    if 0; % not currently used, but paper reports it so saved as documentation
        % short s (+g?) dr i shocks. Use (:,gi) (:,pii) etc. to study other
        % individual shocks.
        
        b2s1 = [err(:,svi)]\err; % shock to gdp and inlation
        xts1(2,:) = -b2s1';
        for indx = 3:H; % find impulse response function
            xts1(indx,:) = xts1(indx-1,:)*A';
        end;
        [sumwpi_s1,sum1wpi_s1,sum1wrnpi_s1,sum1mwrnpi_s1,infl_s1,rn_s1,sums_s1,sumg_s1,sumrpi_s1,omeg_s1] =...
            decode(find_decomp(xts1,pii,rni,gi,svi));
        
        b2r1 = [err*A'*(arn-api)]\err;
        xtr1(2,:) = -b2r1';
        for indx = 3:H; % find impulse response function
            xtr1(indx,:) = xtr1(indx-1,:)*A';
        end;
        [sumwpi_r1,sum1wpi_r1,sum1wrnpi_r1,sum1mwrnpi_r1,infl_r1,rn_r1,sums_r1,sumg_r1,sumrpi_r1,omeg_r1] =...
            decode(find_decomp(xtr1,pii,rni,gi,svi));
        rn1err = err*A'*(arn-api);
        
        % correlation of shocks?
        
        shockts = [err(:,pii) longserr longrerr err(:,gi) err(:,svi) rn1err];
        shocktslbl = {'pi', 'long s','long r', 'g', 's', 'Ern1'};
        disp(shocktslbl);
        disp(corr(shockts))
        
    end 
    
    shockts = [err(:,pii) longserr longrerr err(:,gi) err(:,svi) ];
    shocktslbl = {'pi', 'long s','long r', 'g', 's', 'Ern1'};
    disp(shocktslbl);
    disp(corr(shockts))
    
    % ----------------------------
    % monte carlo standard errors
    % ----------------------------
    
    if Ntrial>0
        
        % save space for standard errors
        
        T = size(err,1);
        
        xt_mc = zeros(H,N,Ntrial); % place to save impulse responses
        xt_nob2_mc = zeros(H,N,Ntrial);
        xt_noA_mc = zeros(H,N,Ntrial);
        xtw_mc = zeros(H,N,Ntrial);
        xtg_mc = zeros(H,N,Ntrial);
        xts_mc = zeros(H,N,Ntrial);
        xtsi_mc = zeros(H,N,Ntrial);
        xtr_mc = zeros(H,N,Ntrial);
        
        x_trial = zeros(T+1,N+1);
        xt_trial = 0*xt;
        xt_nob2_trial = 0*xt;
        xt_noA_trial = 0*xt;
        xtw_trial = 0*xt;
        xtg_trial = 0*xt;
        xts_trial = 0*xt;
        xtsi_trial = 0*xt;
        xtr_trial = 0*xt;
        
        
        decomp_terms = zeros(Ntrial,10);
        decomp_2_terms = zeros(Ntrial,10);
        decomp_2_nob2_terms = zeros(Ntrial,10);
        decomp_2_noA_terms = zeros(Ntrial,10);
        decomp_w_terms = zeros(Ntrial,10);
        decomp_g_terms = zeros(Ntrial,10);
        decomp_s_terms = zeros(Ntrial,10);
        decomp_si_terms = zeros(Ntrial,10);
        decomp_r_terms = zeros(Ntrial,10);
        
        omeg_errors = zeros(9,1); 
        
        b_mc = zeros(size(b,1), size(b,2), Ntrial);
        R2_mc = zeros(size(R2,2),Ntrial);
        
        % following assumes use of sv_x, identity holds exactly so we need to
        % build a singular covaraince matrix. If you use sv, modify
        
        sigma_small = cov(err(:,[rni gi pii vi yi y10i])); % omit s, infer from identity
        C = chol(sigma_small); % C'*C = sigma_small
        
        % hard wired to current variable ordering!
        
        rng(123); % seed random number generator so you get the same answers
        
        nskipped = 0;
        for trial = 1:Ntrial;
            for indx_test=1:1000 ; % sets up a loop for eig < 1
                
                % generate artificial data
                err_small = randn(T,6)*C; % generate errors
                err_s_implied = err_small*[1 -1 -1 -1 0 0]';
                err_t = [err_small(:,1:3) err_s_implied err_small(:,4:6)];
                
                if small_errs;
                    err_t = err_t/10000;
                end;
                
                if check_bs;
                    err_t = err;
                end;
                
                x_trial(1,:) = x(1,:); % generate artificial data
                for tim = 2:T+1; % x is one longer than error
                    x_trial(tim,:) = [1  x_trial(tim-1,:)*b] + [ 0 err_t(tim-1,:)];
                end;
                
                
                % estimate VAR
                % make sure following code is exactly estimation code. Eventually put
                % estimation code in a function to ensure.
                
                x1_trial = x_trial(2:end,2:end);
                b_trial = x_trial(1:end-1,:)\x1_trial;
                A_trial = b_trial(2:end,:)';
                if max(abs(eig(A_trial))) < 1; % only accept convergent A
                    break
                else;
                    nskipped = nskipped+1;
                end
                
            end
            if indx_test>=1000;
                disp('error -- never found A<1')
                stop
            end;
            
            err_trial = x1_trial-x_trial(1:end-1,:)*b_trial;
            
            sigma_trial = cov(err_trial);
            R2_trial = 1-var(err_trial)./var(x1_trial);
            ImA_trial = inv(eye(size(A,1))-A_trial);
            
            % %%%%%%%%%%%%%%%
            % impulse responses
            % Again this code is copied and pasted. Not ideal, make sure it's
            % the same!
            %%%%%%%%%%%%%%%%%
            
            b2_trial = err_trial(:,pii)\err_trial; % regression of other errors on inflation shock
            xt_trial(2,1:N) = b2_trial; % unit inflation shock; orthog last
            for indx = 3:H; % find impulse response function
                xt_trial(indx,1:N) = xt_trial(indx-1,1:N)*A_trial';
            end;
            xt_trial(:,end)=xt_trial(:,rni)-xt_trial(:,pii);
            decomp_terms(trial,:) = find_decomp(xt_trial(:,1:N),pii,rni,gi,svi);
            dt = decomp_terms(trial,:);
            [sumwpi_trial,sum1wpi_trial,sum1wrnpi_trial,sum1mwrnpi_trial,infl_trial,rn_trial,sums_trial,sumg_trial,sumrpi_trial,omeg_trial] = decode(dt);
            
            
            % impulse response with parts turned off
            xt_nob2_trial(2,1:N) = b2; % no variation in b2
            for indx = 3:H; % find impulse response function
                xt_nob2_trial(indx,1:N) = xt_nob2_trial(indx-1,1:N)*A_trial';
            end;
            decomp_2_nob2_terms(trial,:) = find_decomp(xt_nob2_trial(:,1:N),pii,rni,gi,svi);
            xt_nob2_trial(:,end)=xt_nob2_trial(:,rni)-xt_nob2_trial(:,pii);
            
            xt_noA_trial(2,1:N) = b2_trial; % no variation in A
            for indx = 3:H; % find impulse response function
                xt_noA_trial(indx,1:N) = xt_noA_trial(indx-1,1:N)*A_trial';
            end;
            decomp_2_noA_terms(trial,:) = find_decomp(xt_noA_trial(:,1:N),pii,rni,gi,svi);
            xt_noA_trial(:,end)=xt_noA_trial(:,rni)-xt_noA_trial(:,pii);
            
            % long run inflation shock omega j pi t+j
            if do_long_inflation;
                ImwA_trial = inv(eye(size(A_trial,1))-omeg_trial*A_trial);
                b2w_trial = [err_trial*ImwA_trial'*api]\err_trial;
                xtw_trial(2,:) = b2w_trial';
                for indx = 3:H; % find impulse response function
                    xtw_trial(indx,:) = xtw_trial(indx-1,:)*A_trial';
                end;
                % calculate decomposition terms
                decomp_w_terms(trial,:) = find_decomp(xtw_trial(:,1:N),pii,rni,gi,svi);
                xtw_trial(:,end)=xtw_trial(:,rni)-xtw_trial(:,pii);
            end
            
            % AD shock
            
            b2g_trial = [err_trial(:,gi) err_trial(:,pii) ]\err_trial; % shock to gdp and inlation
            xtg_trial(2,:) = -(b2g_trial(1,:)+b2g_trial(2,:))';
            for indx = 3:H; % find impulse response function
                xtg_trial(indx,:) = xtg_trial(indx-1,:)*A_trial';
            end;
            decomp_g_terms(trial,:) = find_decomp(xtg_trial(:,1:N),pii,rni,gi,svi);
            xtg_trial(:,end)=xtg_trial(:,rni)-xtg_trial(:,pii);
            
             % long s+g shock
            
            b2s_trial = [err_trial*ImA_trial'*(asv+ag) ]\err_trial;
            xts_trial(2,:) = -b2s_trial'; % deficit shock
            for indx = 3:H; % find impulse response function
                xts_trial(indx,:) = xts_trial(indx-1,:)*A_trial';
            end;
            decomp_s_terms(trial,:) = find_decomp(xts_trial(:,1:N),pii,rni,gi,svi);
            xts_trial(:,end)=xts_trial(:,rni)-xts_trial(:,pii); % keep rn - pi separately in monte carlo
            
            
            % long s+g no i shock
            
            b2si_trial = [err_trial*ImA_trial'*(asv+ag) err_trial(:,yi) err_trial*A_trial'*ay3 ]\err_trial; % hold one period rate for two years
            xtsi_trial(2,:) = -b2si_trial(1,:)'; % deficit shock with no change in interest rate
            for indx = 3:H; % find impulse response function
                xtsi_trial(indx,:) = xtsi_trial(indx-1,:)*A_trial';
            end;
            decomp_si_terms(trial,:) = find_decomp(xtsi_trial,pii,rni,gi,svi);
            
            % dr shock            
            
            omeg_r_old_trial = omeg_trial;
            maxomiter = 20;
            for iteration = 1:maxomiter;
                ImwAr_trial = omeg_r_old_trial*A_trial*inv((eye(size(A_trial,1))-omeg_r_old_trial*A_trial));
                ImAr_trial = A_trial*inv((eye(size(A_trial,1))-A_trial));
                b2r_trial = [err_trial*(ImAr_trial'-ImwAr_trial')*(arn-api)]\err_trial;
                xtr_trial(2,:) = -b2r_trial'; % discount rate shock
                for indx = 3:H; % find impulse response function
                    xtr_trial(indx,:) = xtr_trial(indx-1,:)*A_trial';
                end;
                [sumwpi_r_trial,sum1wpi_r_trial,sum1wrnpi_r_trial,sum1mwrnpi_r_trial,infl_r_trial,rn_r_trial,...
                    sums_r_trial,sumg_r_trial,sumrpi_r_trial,omeg_r_trial] = decode(find_decomp(xtr_trial,pii,rni,gi,svi));
                %disp([iteration omeg_r_old omeg_r sum1mwrnpi_r]);
                if abs(sum1mwrnpi_r_trial - (-1.00)) < 0.0025;
                    break
                else
                    omeg_r_old_trial = omeg_r_trial;
                end
            end
            if iteration >=maxomiter;
                disp('MC#: omega did not converge. trial iteration new and old omega sum should be 1');
                disp([trial iteration omeg_r_old_trial omeg_r_trial sum1mwrnpi_r_trial]);
            end
            
            decomp_r_terms(trial,:) = find_decomp(xtr_trial(:,1:N),pii,rni,gi,svi);
            xtr_trial(:,end)=xtr_trial(:,rni)-xtr_trial(:,pii);
            
            %save responses
            
            xt_mc(:,:,trial) = xt_trial;
            xt_nob2_mc(:,:,trial) = xt_nob2_trial;
            xt_noA_mc(:,:,trial) = xt_noA_trial;
            xtw_mc(:,:,trial) = xtw_trial;
            xtg_mc(:,:,trial) = xtg_trial;
            xts_mc(:,:,trial) = xts_trial;
            xtsi_mc(:,:,trial) = xtsi_trial;
            xtr_mc(:,:,trial) = xtr_trial;
            
            % save estimates
            b_mc(:,:,trial) = [b_trial(1,:);A_trial'];
            R2_mc(:,trial) = R2_trial';
            
        end
        
        disp('number of skipped draws due to |A|>1 out of')
        disp([nskipped Ntrial])
        
        b_mean = mean(b_mc,3);
        b_se = std(b_mc,0,3);
        
        R2_mean = mean(R2_mc,2)';
        R2_se = std(R2_mc,0,2)';
        
        xt_mean = mean(xt_mc,3);
        xt_se = std(xt_mc,0,3)';
        xt_nob2_mean = mean(xt_nob2_mc,3);
        xt_nob2_se = std(xt_nob2_mc,0,3)';
        xt_noA_mean = mean(xt_noA_mc,3);
        xt_noA_se = std(xt_noA_mc,0,3)';
        xtw_mean = mean(xtw_mc,3);
        xtw_se = std(xtw_mc,0,3)';
        xtg_mean = mean(xtg_mc,3);
        xtg_se = std(xtg_mc,0,3)';
        xts_mean = mean(xts_mc,3);
        xts_se = std(xts_mc,0,3)';
        xtsi_mean = mean(xtsi_mc,3);
        xtsi_se = std(xtsi_mc,0,3)';
        xtr_mean = mean(xtr_mc,3);
        xtr_se = std(xtr_mc,0,3)';
        
        xt_mc = sort(xt_mc,3);
        xt_nob2_mc = sort(xt_nob2_mc,3);
        xt_noA_mc = sort(xt_noA_mc,3);
        xtw_mc = sort(xtw_mc,3);
        xtg_mc = sort(xtg_mc,3);
        xts_mc = sort(xts_mc,3);
        xtsi_mc = sort(xtsi_mc,3);
        xtr_mc = sort(xtr_mc,3);
        
        indx25 = floor(Ntrial*0.25)+1; % index of 25th and 75th percentiles. round to make bigger tails
        indx50 = floor((Ntrial)/2);% round median down -- no choice!
        indx75 = floor(Ntrial*0.75)-1; % symmetrically, round to bigger tails
        
        xt_25 = xt_mc(:,:,indx25);
        xt_50 = xt_mc(:,:,indx50);
        xt_75 = xt_mc(:,:,indx75);
        xt_nob2_25 = xt_nob2_mc(:,:,indx25);
        xt_nob2_50 = xt_nob2_mc(:,:,indx50);
        xt_nob2_75 = xt_nob2_mc(:,:,indx75);
        xt_noA_25 = xt_noA_mc(:,:,indx25);
        xt_noA_50 = xt_noA_mc(:,:,indx50);
        xt_noA_75 = xt_noA_mc(:,:,indx75);
        xtw_25 = xtw_mc(:,:,indx25);
        xtw_50 = xtw_mc(:,:,indx50);
        xtw_75 = xtw_mc(:,:,indx75);
        xtg_25 = xtg_mc(:,:,indx25);
        xtg_50 = xtg_mc(:,:,indx50);
        xtg_75 = xtg_mc(:,:,indx75);
        xts_25 = xts_mc(:,:,indx25);
        xts_50 = xts_mc(:,:,indx50);
        xts_75 = xts_mc(:,:,indx75);
        xtsi_25 = xtsi_mc(:,:,indx25);
        xtsi_50 = xtsi_mc(:,:,indx50);
        xtsi_75 = xtsi_mc(:,:,indx75);
        xtr_25 = xtr_mc(:,:,indx25);
        xtr_50 = xtr_mc(:,:,indx50);
        xtf_75 = xtr_mc(:,:,indx75);
        
        % save the terms for covariance analysis
        decomp_terms_mean = mean(decomp_terms,1);
        decomp_terms_se = std(decomp_terms,1);
        decomp_terms = sort(decomp_terms,1);
        decomp_terms_25 = decomp_terms(indx25,:);
        decomp_terms_50 = decomp_terms(indx50,:);
        decomp_terms_75 = decomp_terms(indx75,:);
        
        decomp_2_nob2_terms_mean = mean(decomp_2_nob2_terms,1);
        decomp_2_nob2_terms_se = std(decomp_2_nob2_terms,1);
        decomp_2_nob2_terms = sort(decomp_2_nob2_terms,1);
        decomp_2_nob2_terms_25 = decomp_2_nob2_terms(indx25,:);
        decomp_2_nob2_terms_50 = decomp_2_nob2_terms(indx50,:);
        decomp_2_nob2_terms_75 = decomp_2_nob2_terms(indx75,:);
        
        decomp_2_noA_terms_mean = mean(decomp_2_noA_terms,1);
        decomp_2_noA_terms_se = std(decomp_2_noA_terms,1);
        decomp_2_noA_terms = sort(decomp_2_noA_terms,1);
        decomp_2_noA_terms_25 = decomp_2_noA_terms(indx25,:);
        decomp_2_noA_terms_50 = decomp_2_noA_terms(indx50,:);
        decomp_2_noA_terms_75 = decomp_2_noA_terms(indx75,:);
        
        decomp_w_terms_mean = mean(decomp_w_terms,1);
        decomp_w_terms_se = std(decomp_w_terms,1);
        decomp_w_terms = sort(decomp_w_terms,1);
        decomp_w_terms_25 = decomp_w_terms(indx25,:);
        decomp_w_terms_50 = decomp_w_terms(indx50,:);
        decomp_w_terms_75 = decomp_w_terms(indx75,:);
        
        decomp_g_terms_mean = mean(decomp_g_terms,1);
        decomp_g_terms_se = std(decomp_g_terms,1);
        decomp_g_terms = sort(decomp_g_terms,1);
        decomp_g_terms_25 = decomp_g_terms(indx25,:);
        decomp_g_terms_50 = decomp_g_terms(indx50,:);
        decomp_g_terms_75 = decomp_g_terms(indx75,:);
        
        decomp_s_terms_mean = mean(decomp_s_terms,1);
        decomp_s_terms_se = std(decomp_s_terms,1);
        decomp_s_terms = sort(decomp_s_terms,1);
        decomp_s_terms_25 = decomp_s_terms(indx25,:);
        decomp_s_terms_50 = decomp_s_terms(indx50,:);
        decomp_s_terms_75 = decomp_s_terms(indx75,:);
        
        decomp_si_terms_mean = mean(decomp_si_terms,1);
        decomp_si_terms_se = std(decomp_si_terms,1);
        decomp_si_terms = sort(decomp_si_terms,1);
        decomp_si_terms_25 = decomp_si_terms(indx25,:);
        decomp_si_terms_50 = decomp_si_terms(indx50,:);
        decomp_si_terms_75 = decomp_si_terms(indx75,:);
      
        decomp_r_terms_mean = mean(decomp_r_terms,1);
        decomp_r_terms_se = std(decomp_r_terms,1);
        decomp_r_terms = sort(decomp_r_terms,1);
        decomp_r_terms_25 = decomp_r_terms(indx25,:);
        decomp_r_terms_50 = decomp_r_terms(indx50,:);
        decomp_r_terms_75 = decomp_r_terms(indx75,:);
        
        
        
    end; % ends if Ntrial>0 that stops out monte carlo
    
    if ~save_last_bs;
        if stdate == 1947
            save do_bs_save_5;
        elseif stdate == 1930; 
            save do_bs_save_30;
        elseif stdate == 1980
            save do_bs_save_80; 
        else
            disp('you shouldn''t be here')
        end
    end;
else % if not do_bs
    if stdate == 1947
        load do_bs_save_5
    elseif stdate == 1930
        load do_bs_save_30;
    elseif stdate == 1980
        load do_bs_save_80;
    end
end;

%%%%%%%%%
% Additional calculations
%%%%%%%%%

%%%%%%%%%%
% output
%%%%%%%%%

% NEW TABLES
script_make_horiz_table;

% plot surplus
set(0,'DefaultLegendAutoUpdate','off');
svlin = figure;
p2 = plot(datenum(yr_a,12,31) ,[ svt ],'linewidth',2);
hold on;
p3 = plot(datenum(yr_a,12,31) ,[ syt/exp(mean(vt))],'linewidth',2);
p1 = plot(datenum(yr_a,12,31) ,[ svt_x ],'linewidth',2);
p4 = plot(datenum(1930,12,31)*[1;1],[0.3;-0.83],'--k');
p5 = plot(datenum(yr_a,12,31),0*yr_a,'-k');
%title('Surplus/value ratio')
legend([p1 p2 p3],{'Linear, s_t','sv_t','sy_t/e^v'},'location','best')
%legend('boxoff')
recessionplot;
% old axes, show wwii
%axis([datenum(1929,21,31) datenum(2020,12,31) -0.85 0.3])
axis([datenum(1929,21,31) datenum(2020,12,31) -0.35 0.2])
datetick('x','keeplimits');
xlabel('Date')
ylabel('Surplus/value of debt');
print(svlin,'sv_linear_comparison.eps','-depsc');

% print VAR

if tex_files
    if stdate == 1947;
        VAR_ID = fopen('VAR.tex','w');
        caption = 'OLS VAR estimate. Sample 1947-2018. ';
        texlabel='VAR';
    elseif stdate == 1930; 
        VAR_ID = fopen('VAR_full.tex','w');
        caption = 'OLS VAR estimate. Sample 1930-2018. '
        texlabel='VAR_full';
    elseif  stdate == 1980; 
        VAR_ID = fopen('VAR_80.tex','w');
        caption = 'OLS VAR estimate. Sample 1980-2018. '
        texlabel='VAR_80';
    else
        disp('tex files on but no valid start date')
        stop
    end
else
    VAR_ID = 1
    caption = 'OLS VAR estimate.'
    texlabel='VAR';
end
if Ntrial == 0; 
    b_se = 0; 
    R2_se = 0; 
    xt_se = 0*xt; 
end; 
f_printvar(VAR_ID,b,b_se,sigma,R2,R2_se,lbls,texlabel,caption,pii);
if tex_files
    fclose(VAR_ID);
end

% print correlation matrix and b2
f_print_cormat; 


if 0
     % scatterplot of s and pi innovations
    figure
    plot(err(:,svi),err(:,pii),'.','markersize',0.001) % just to set scale
    text(err(:,svi),err(:,pii),num2str(yr_a(stind+1:end),'%4.0i'),'fontsize',12);
    xlabel('s innovation','fontsize',16)
    ylabel('\pi innovation','fontsize',16)
end

% plot impulse response function 

fig_ir1 = figure;
colors = get(fig_ir1,'defaultaxescolororder');
xplot = plot((0:H-1)',xt(:,[pii svi gi ]) ,'linewidth',2); 
hold on
rnplotl = plot((2:H-1)',xt(3:end,rni)-xt(3:end,pii),'linewidth',2);
plot(ones(1,3),[xt(2,[pii gi svi]) ],'v');
plot((0:H-1)',zeros(H,1),'-k');
if (stdate == 1947);
    text(2.3,xt(2+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(4,xt(4+1,svi)-0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(1.1,xt(1+1,gi)-0.05,[char(lbls(gi)) ', \Sigma=' num2str(sumg,'%4.2f')],'color',colors(3,:),'fontsize',16);
    text(6,xt(6+1,rni)-xt(6+1,pii)+0.10,...
        ['r, \Sigma=' num2str(sumrpi,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    axis([0 15 -inf 1.05]);
end;
if (stdate == 1930);
    text(2.3,xt(2+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(5,xt(5+1,svi)-0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(2.1,xt(2+1,gi)+0.07,[char(lbls(gi)) ', \Sigma=' num2str(sumg,'%4.2f')],'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(4.2,xt(4+1,rni)-xt(4+1,pii)-0.07,...
        ['r, \Sigma=' num2str(sumrpi,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    axis([0 15 -inf 1.05]);
end;
if (stdate == 1980);
    text(2.3,xt(2+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(5,xt(5+1,svi)-0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(3.1,xt(3+1,gi)-0.07,[char(lbls(gi)) ', \Sigma=' num2str(sumg,'%4.2f')],'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(2.5,xt(2+1,rni)-xt(2+1,pii)-0.07,...
        ['r, \Sigma=' num2str(sumrpi,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    axis([0 15 -inf 1.3]);
end;

xlabel('Years');
ylabel('Percent');
%title('Response to inflation shock')
if stdate == 1947;
    print( fig_ir1, 'impulse-response_1.eps', '-depsc');
    print( fig_ir1, 'impulse-response_1_png.png', '-dpng');
elseif stdate == 1930;
    print( fig_ir1, 'impulse-response_1_full.eps', '-depsc');
elseif stdate == 1980;
    print( fig_ir1, 'impulse-response_1_80.eps', '-depsc');
end

fig_ir2 = figure;
colors = get(fig_ir2,'defaultaxescolororder');
xplot = plot((0:H-1)',xt(:,[pii svi vi]) ,'linewidth',2); 
hold on
plot((0:H-1)',zeros(H,1),'-k');
if stdate == 1947;
    text(2.3,xt(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xt(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(4,xt(4+1,svi)-0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
end
if (stdate == 1930);
    text(2.3,xt(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xt(6+1,vi)-0.05,lbls(vi),'color',colors(3,:),'fontsize',16);
    text(4,xt(4+1,svi)-0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
end
if (stdate == 1980);
    text(2.3,xt(2+1,pii)+0.2,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xt(6+1,vi)-0.05,lbls(vi),'color',colors(3,:),'fontsize',16);
    text(4,xt(4+1,svi)-0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums,'%4.2f')],'color',colors(2,:),'fontsize',16);
end

axis([0 25 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
if stdate == 1947;
    print( fig_ir2, 'impulse-response_2.eps', '-depsc');
    print( fig_ir2, 'impulse-response_2_png.png', '-dpng');
elseif stdate == 1930;
    print( fig_ir2, 'impulse-response_2_full.eps', '-depsc');
elseif stdate == 1980;
    print( fig_ir2, 'impulse-response_2_80.eps', '-depsc');
end

fig_ir3 = figure;
xplot = plot((0:H-1)',xt(:,[pii rni yi y10i]),'linewidth',2); 
hold on
plot(ones(1,4),xt(2,[pii rni yi y10i]),'v');
plot((0:H-1)',zeros(H,1),'-k');
if stdate == 1947;
    text(2,xt(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(5,xt(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(6,xt(6+1,yi),lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(7,xt(7+1,y10i),lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    %title('Response to inflation shock')
    axis([0 10 -inf 1.1]);
end;
if (stdate == 1930);
    text(2.5,xt(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16);
    text(3,xt(3+1,rni)-0.04,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(2,xt(2+1,yi)+0.05,lbls(yi),'color',colors(3,:),'fontsize',16);
    text(2,xt(2+1,y10i)-0.02,lbls(y10i),'color',colors(4,:),'fontsize',16);
    %title('Response to inflation shock')
    axis([0 10 -0.2 1.1]);
end;
if (stdate == 1980);
    text(2,xt(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'horizontalalignment','r');
    text(3,xt(3+1,rni)-0.04,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(2,xt(2+1,yi)+0.05,lbls(yi),'color',colors(3,:),'fontsize',16);
    text(2,xt(2+1,y10i)-0.02,lbls(y10i),'color',colors(4,:),'fontsize',16);
    %title('Response to inflation shock')
    axis([0 10 -0.2 1.1]);
end;
xlabel('Years')
ylabel('Percent')
if stdate == 1947;
    print(fig_ir3, 'impulse-response_3.eps', '-depsc');
    print(fig_ir3, 'impulse-response_3_png.png', '-dpng');
elseif stdate == 1930; 
    print(fig_ir3, 'impulse-response_3_full.eps', '-depsc');
elseif stdate == 1980; 
    print(fig_ir3, 'impulse-response_3_80.eps', '-depsc');
end
    

% --------------------------------
% Response to other shocks 
% ---------------------------------------

% AD shock 

fig_ir_g1 = figure;
colors = get(fig_ir_g1,'defaultaxescolororder');
xplot = plot((0:H-1)',xtg(:,[pii svi gi ]) ,'linewidth',2); 
hold on
rnplotl = plot((2:H-1)',xtg(3:end,rni)-xtg(3:end,pii),'linewidth',2);
plot(ones(1,3),[xtg(2,[pii gi svi]) ],'v');
plot((0:H-1)',zeros(H,1),'-k');
if stdate == 1947;
    text(3.3,xtg(3+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_g, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(7,xtg(7+1,svi)+0.05,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(4.2,xtg(4+1,gi)+0.07,[char(lbls(gi)) ', \Sigma=' num2str(sumg_g,'%4.2f')],...
        'color',colors(3,:),'fontsize',16);
    text(8,xtg(8+1,rni)-xtg(8+1,pii)-0.1,...
        ['r, \Sigma=' num2str(sumrpi_g,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_g,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16);
    axis([0 15 -inf inf]);
end;
if (stdate == 1930);
    text(3.3,xtg(3+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_g, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(4,xtg(4+1,svi)+0.075,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(2,xtg(2+1,gi),[char(lbls(gi)) ', \Sigma=' num2str(sumg_g,'%4.2f')],...
        'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(4,xtg(4+1,rni)-xtg(3+1,pii),...
        ['r, \Sigma=' num2str(sumrpi_g,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_g,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    axis([0 15 -1.1 0.9]);
end;
if (stdate == 1980);
    text(3.3,xtg(3+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_g, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(4,xtg(4+1,svi)+0.075,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(2,xtg(2+1,gi),[char(lbls(gi)) ', \Sigma=' num2str(sumg_g,'%4.2f')],...
        'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(4,xtg(4+1,rni)-xtg(3+1,pii),...
        ['r, \Sigma=' num2str(sumrpi_g,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_g,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    axis([0 15 -1.1 0.9]);
end;
xlabel('Years')
ylabel('Percent')
if stdate == 1947;
    print( fig_ir_g1, 'impulse-response_g1.eps', '-depsc');
    print( fig_ir_g1, 'impulse-response_g1_png.png', '-dpng');
elseif stdate == 1930;
    print( fig_ir_g1, 'impulse-response_g1_full.eps', '-depsc');
elseif stdate == 1980;
    print( fig_ir_g1, 'impulse-response_g1_80.eps', '-depsc');
end

fig_ir2 = figure;
colors = get(fig_ir2,'defaultaxescolororder');
xplot = plot((0:H-1)',xtg(:,[pii svi vi]) ,'linewidth',2); 
hold on
plot((0:H-1)',zeros(H,1),'-k');
if stdate == 1947;
    text(4.3,xtg(4+1,pii)-0.05,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xtg(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(5,xtg(5+1,svi)+0.5,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    axis([0 15 -inf inf]);
end;
if (stdate == 1930);
    text(4.3,xtg(4+1,pii)-0.05,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xtg(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(4.2,xtg(4+1,svi)+0.15,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    axis([0 15 -inf inf]);
end;
if (stdate == 1980);
    text(4.3,xtg(4+1,pii)-0.05,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xtg(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(4.2,xtg(4+1,svi)+0.15,[char(lbls(svi)) ', \Sigma=' num2str(sums_g,'%4.2f')],'color',colors(2,:),'fontsize',16);
    axis([0 15 -inf inf]);
end;

xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
if stdate == 1947;
    print( fig_ir2, 'impulse-response_g2.eps', '-depsc');
    print( fig_ir2, 'impulse-response_g2_png.png', '-dpng');
elseif stdate == 1930;
    print( fig_ir2, 'impulse-response_g2_full.eps', '-depsc');
elseif stdate == 1980;
    print( fig_ir2, 'impulse-response_g2_80.eps', '-depsc');
end

fig_ir3 = figure;
xplot = plot((0:H-1)',xtg(:,[pii rni yi y10i]),'linewidth',2); 
hold on
plot(ones(1,4),xtg(2,[pii rni yi y10i]),'v');
plot((0:H-1)',zeros(H,1),'-k');
if stdate == 1947;
    text(5,xtg(5+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(5,xtg(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(6,xtg(6+1,yi),lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(7,xtg(7+1,y10i),lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    %title('Response to inflation shock')
    axis([0 10 -inf inf]);
end;
if (stdate == 1930);
    text(2,xtg(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(1.5,xtg(1+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16);
    text(1,xtg(1+1,yi),lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(3,xtg(3+1,y10i),lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    %title('Response to inflation shock')
    axis([0 10 -1.1 0.3]);
end
if (stdate == 1980);
    text(2,xtg(2+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(1.5,xtg(1+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16);
    text(1,xtg(1+1,yi),lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
    text(3,xtg(3+1,y10i),lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
    %title('Response to inflation shock')
    axis([0 10 -1.1 0.3]);
end;
xlabel('Years')
ylabel('Percent')
if stdate == 1947;
    print(fig_ir3, 'impulse-response_g3.eps', '-depsc');
    %print(fig_ir3, 'impulse-response_g3_png.png', '-dpng');
elseif stdate == 1930;
    print(fig_ir3, 'impulse-response_g3_full.eps', '-depsc');
elseif stdate == 1980;
    print(fig_ir3, 'impulse-response_g3_full.eps', '-depsc');
end


%%%%%
% WWII data
%%%%%%

if (stdate == 1930) & (enddate == 2018); 
figure
plot(datenum(yr_a(2:end),6,1) ,xl(:,[svi]),'-b','linewidth',2);
hold on;
plot(datenum(yr_a(2:end),6,1) ,0*xl(:,[svi])+mean(xl(:,[svi])),'--b','linewidth',2);
plot(datenum(yr_a(2:end),6,1) ,xl(:,[pii]),'-r','linewidth',2);
plot(datenum(yr_a(2:end),6,1) ,0*xl(:,[pii])+mean(xl(:,[pii])),'--r','linewidth',2);
plot(datenum(yr_a(2:end),6,1) ,0*xl(:,[svi]),'-k');
%legend('s','mean s','\pi','mean \pi','location','best');
%axis([datenum(1930,21,31) datenum(2020,12,31) -inf inf])
axis([datenum(1930,1,1) datenum(1955,1,1) -inf inf])
datetick('x','keeplimits')
annotation('arrow',[0.5 0.57],[0.82 0.45])
text(datenum(1941,6,1),0.1,'\pi','fontsize',16,'color','r');
text(datenum(1943,6,1),-0.5,'s','fontsize',16,'color','b')
print -depsc2 WWIIspi.eps
end;

if stdate == 1947; % only do these plots for main sample case. Mostly not pretty and file names not set

% surplus shock 

fig_ir_s1 = figure;
colors = get(fig_ir_s1,'defaultaxescolororder');
xplot = plot((0:H-1)',xts(:,[pii svi gi ]) ,'linewidth',2); 
hold on
rnplotl = plot((2:H-1)',xts(3:end,rni)-xts(3:end,pii),'linewidth',2);
plot(ones(1,3),[xts(2,[pii gi svi]) ],'v');
plot((0:H-1)',zeros(H,1),'-k');
text(5,xts(5+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_s, '%4.2f')],'color',colors(1,:),'fontsize',16);
text(3,xts(3+1,svi)-0.2,[char(lbls(svi)) ', \Sigma=' num2str(sums_s,'%4.2f')],'color',colors(2,:),'fontsize',16);
text(3.5,xts(3+1,gi)+0.02,[char(lbls(gi)) ', \Sigma=' num2str(sumg_s,'%4.2f')],'color',colors(3,:),'fontsize',16,'HorizontalAlignment','r');
text(8,xts(8+1,rni)-xts(8+1,pii)-0.02,...
   ['r, \Sigma=' num2str(sumrpi_s,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_s,'%4.2f') ],...
'color',colors(4,:),'fontsize',16);
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
if (stdate == 1947);
    print( fig_ir_s1, 'impulse-response_s1.eps', '-depsc');
    print( fig_ir_s1, 'impulse-response_s1_png.png', '-dpng');
end;

fig_ir2 = figure;
colors = get(fig_ir2,'defaultaxescolororder');
xplot = plot((0:H-1)',xts(:,[pii svi vi]) ,'linewidth',2); 
hold on
plot((0:H-1)',zeros(H,1),'-k');
text(4.3,xts(4+1,pii)-0.05,lbls(pii),'color',colors(1,:),'fontsize',16);
text(6.3,xts(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
text(5,xts(5+1,svi)+0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums_s,'%4.2f')],'color',colors(2,:),'fontsize',16);
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
%print( fig_ir2, 'impulse-response_s2.eps', '-depsc');
%print( fig_ir2, 'impulse-response_s2_png.png', '-dpng');


fig_ir3 = figure;
xplot = plot((0:H-1)',xts(:,[pii rni yi y10i]),'linewidth',2); 
hold on
plot(ones(1,4),xts(2,[pii rni yi y10i]),'v');
plot((0:H-1)',zeros(H,1),'-k');
text(7,xts(7+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
text(5,xts(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(1.5,xts(1+1,rni)-0.1,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(6,xts(6+1,yi)+0.01,lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
text(7,xts(7+1,y10i)-0.01,lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
%title('Response to inflation shock')
axis([0 10 -inf inf]);
xlabel('Years')
ylabel('Percent')
if (stdate == 1947);
    print(fig_ir3, 'impulse-response_s3.eps', '-depsc');
    print(fig_ir3, 'impulse-response_s3_png.png', '-dpng');
end


% surplus shock -- no interest rate response

fig_ir_s1 = figure;
colors = get(fig_ir_s1,'defaultaxescolororder');
xplot = plot((0:H-1)',xtsi(:,[pii svi gi ]) ,'linewidth',2); 
hold on
rnplotl = plot((2:H-1)',xtsi(3:end,rni)-xtsi(3:end,pii),'linewidth',2);
plot(ones(1,3),[xtsi(2,[pii gi svi]) ],'v');
plot((0:H-1)',zeros(H,1),'-k');
text(2.3,xtsi(2+1,pii)+.02,[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_si, '%4.2f')],'color',colors(1,:),'fontsize',16);
text(2.3,xtsi(2+1,svi),[char(lbls(svi)) ', \Sigma=' num2str(sums_si,'%4.2f')],'color',colors(2,:),'fontsize',16);
text(3.2,xtsi(3+1,gi)+0.04,[char(lbls(gi)) ', \Sigma=' num2str(sumg_si,'%4.2f')],'color',colors(3,:),'fontsize',16,'HorizontalAlignment','r');
text(2.3,xtsi(2+1,rni)-xtsi(2+1,pii),...
   ['r, \Sigma=' num2str(sumrpi_si,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_si,'%4.2f') ],...
'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
if (stdate == 1947);
    print( fig_ir_s1, 'impulse-response_si1.eps', '-depsc');
    print( fig_ir_s1, 'impulse-response_si1_png.png', '-dpng');
end

fig_ir2 = figure;
colors = get(fig_ir2,'defaultaxescolororder');
xplot = plot((0:H-1)',xtsi(:,[pii svi vi]) ,'linewidth',2);
hold on
plot((0:H-1)',zeros(H,1),'-k');
text(4.3,xtsi(4+1,pii)-0.05,lbls(pii),'color',colors(1,:),'fontsize',16);
text(6.3,xtsi(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
text(5,xtsi(5+1,svi)+0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums_si,'%4.2f')],'color',colors(2,:),'fontsize',16);
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
%print( fig_ir2, 'impulse-response_s2.eps', '-depsc');
%print( fig_ir2, 'impulse-response_s2_png.png', '-dpng');


fig_ir3 = figure;
xplot = plot((0:H-1)',xtsi(:,[pii rni yi y10i]),'linewidth',2);
hold on
plot(ones(1,4),xtsi(2,[pii rni yi y10i]),'v');
plot((0:H-1)',zeros(H,1),'-k');
text(7,xtsi(7+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
text(5,xtsi(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(1.5,xtsi(1+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(6,xtsi(6+1,yi),lbls(yi),'color',colors(3,:),'fontsize',16,'backgroundcolor','w');
text(7,xtsi(7+1,y10i),lbls(y10i),'color',colors(4,:),'fontsize',16,'backgroundcolor','w');
%title('Response to inflation shock')
axis([0 10 -inf inf]);
xlabel('Years')
ylabel('Percent')
if (stdate == 1947);
    print(fig_ir3, 'impulse-response_si3.eps', '-depsc');
    print(fig_ir3, 'impulse-response_si3_png.png', '-dpng');
end

% Discount rate shock

fig_ir_s1 = figure;
colors = get(fig_ir_s1,'defaultaxescolororder');
xplot = plot((0:H-1)',xtr(:,[pii svi gi ]) ,'linewidth',2);
hold on
rnplotl = plot((2:H-1)',xtr(3:end,rni)-xtr(3:end,pii),'linewidth',2);
plot(ones(1,3),[xtr(2,[pii gi svi]) ],'v');
plot((0:H-1)',zeros(H,1),'-k');
text(6,xtr(6+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_r, '%4.2f')],'color',colors(1,:),'fontsize',16);
text(3,xtr(3+1,svi)-0.2,[char(lbls(svi)) ', \Sigma=' num2str(sums_r,'%4.2f')],'color',colors(2,:),'fontsize',16);
text(3.5,xtr(3+1,gi)+0.02,[char(lbls(gi)) ', \Sigma=' num2str(sumg_r,'%4.2f')],'color',colors(3,:),'fontsize',16,'HorizontalAlignment','r');
text(8,xtr(8+1,rni)-xtr(8+1,pii)-0.02,...
    ['r, \Sigma=' num2str(sumrpi_r,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_r,'%4.2f') ],...
    'color',colors(4,:),'fontsize',16);
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
if (stdate == 1947);
    print( fig_ir_s1, 'impulse-response_r1.eps', '-depsc');
    print( fig_ir_s1, 'impulse-response_r1_png.png', '-dpng');
end;

fig_ir2 = figure;
colors = get(fig_ir2,'defaultaxescolororder');
xplot = plot((0:H-1)',xtr(:,[pii svi vi]) ,'linewidth',2);
hold on
plot((0:H-1)',zeros(H,1),'-k');
text(5,xtr(5+1,pii)-0.005,lbls(pii),'color',colors(1,:),'fontsize',16);
text(6.3,xtr(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
text(5,xtr(5+1,svi)+0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums_r,'%4.2f')],'color',colors(2,:),'fontsize',16);
axis([0 15 -inf inf]);
xlabel('Years')
ylabel('Percent')
%title('Response to inflation shock')
%print( fig_ir2, 'impulse-response_r2.eps', '-depsc');
%print( fig_ir2, 'impulse-response_r2_png.png', '-dpng');


fig_ir3 = figure;
xplot = plot((0:H-1)',xtr(:,[pii rni yi y10i]),'linewidth',2);
hold on
plot(ones(1,4),xtr(2,[pii rni yi y10i]),'v');
plot((0:H-1)',zeros(H,1),'-k');
text(5,xtr(5+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
text(5,xtr(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(1.5,xtr(1+1,rni)-0.05,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
text(6,xtr(6+1,yi)+0.01,lbls(yi),'color',colors(3,:),'fontsize',16);
text(7,xtr(7+1,y10i)-0.01,lbls(y10i),'color',colors(4,:),'fontsize',16);
%title('Response to inflation shock')
axis([0 10 -inf inf]);
xlabel('Years')
ylabel('Percent')
if (stdate == 1947);
    print(fig_ir3, 'impulse-response_r3.eps', '-depsc');
    print(fig_ir3, 'impulse-response_r3_png.png', '-dpng');
end;

% one period surplus shock
if 0;
    fig_ir_s1 = figure;
    colors = get(fig_ir_s1,'defaultaxescolororder');
    xplot = plot((0:H-1)',xts1(:,[pii svi gi ]) ,'linewidth',2);
    hold on
    rnplotl = plot((2:H-1)',xts1(3:end,rni)-xts1(3:end,pii),'linewidth',2);
    plot(ones(1,3),[xts1(2,[pii gi svi]) ],'v');
    plot((0:H-1)',zeros(H,1),'-k');
    text(6,xts1(6+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_s1, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(3,xts1(3+1,svi)-0.2,[char(lbls(svi)) ', \Sigma=' num2str(sums_s1,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(3.5,xts1(3+1,gi)+0.02,[char(lbls(gi)) ', \Sigma=' num2str(sumg_s1,'%4.2f')],'color',colors(3,:),'fontsize',16,'HorizontalAlignment','r');
    text(8,xts1(8+1,rni)-xtr(8+1,pii)-0.02,...
        ['r, \Sigma=' num2str(sumrpi_s1,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_s1,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16);
    axis([0 15 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %title('Response to inflation shock')
    %print( fig_ir_s1, 'impulse-response_s11.eps', '-depsc');
    %print( fig_ir_s1, 'impulse-response_s11_png.png', '-dpng');
    
    
    fig_ir2 = figure;
    colors = get(fig_ir2,'defaultaxescolororder');
    xplot = plot((0:H-1)',xts1(:,[pii svi vi]) ,'linewidth',2);
    hold on
    plot((0:H-1)',zeros(H,1),'-k');
    text(5,xts1(5+1,pii)-0.005,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xts1(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(5,xts1(5+1,svi)+0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums_s1,'%4.2f')],'color',colors(2,:),'fontsize',16);
    axis([0 15 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %title('Response to inflation shock')
    %print( fig_ir2, 'impulse-response_s12.eps', '-depsc');
    %print( fig_ir2, 'impulse-response_s12_png.png', '-dpng');
    
    
    fig_ir3 = figure;
    xplot = plot((0:H-1)',xts1(:,[pii rni yi y10i]),'linewidth',2);
    hold on
    plot(ones(1,4),xts1(2,[pii rni yi y10i]),'v');
    plot((0:H-1)',zeros(H,1),'-k');
    text(5,xts1(5+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(5,xts1(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(1.5,xts1(1+1,rni)-0.05,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(6,xts1(6+1,yi)+0.01,lbls(yi),'color',colors(3,:),'fontsize',16);
    text(7,xts1(7+1,y10i)-0.01,lbls(y10i),'color',colors(4,:),'fontsize',16);
    %title('Response to inflation shock')
    axis([0 10 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %print(fig_ir3, 'impulse-response_s13.eps', '-depsc');
    %print(fig_ir3, 'impulse-response_s13_png.png', '-dpng');
    
    
    % one period discount rate shock
    
    fig_ir_s1 = figure;
    colors = get(fig_ir_s1,'defaultaxescolororder');
    xplot = plot((0:H-1)',xtr1(:,[pii svi gi ]) ,'linewidth',2);
    hold on
    rnplotl = plot((2:H-1)',xtr1(3:end,rni)-xtr1(3:end,pii),'linewidth',2);
    plot(ones(1,3),[xtr1(2,[pii gi svi]) ],'v');
    plot((0:H-1)',zeros(H,1),'-k');
    text(6,xtr1(6+1,pii),[char(lbls(pii)) ', \Sigma\omega^j=' num2str(sumwpi_r1, '%4.2f')],'color',colors(1,:),'fontsize',16);
    text(3,xtr1(3+1,svi)-0.2,[char(lbls(svi)) ', \Sigma=' num2str(sums_r1,'%4.2f')],'color',colors(2,:),'fontsize',16);
    text(3.5,xtr1(3+1,gi)+0.02,[char(lbls(gi)) ', \Sigma=' num2str(sumg_r1,'%4.2f')],'color',colors(3,:),'fontsize',16,'HorizontalAlignment','r');
    text(8,xtr1(8+1,rni)-xtr(8+1,pii)-0.02,...
        ['r, \Sigma=' num2str(sumrpi_r1,'%4.2f') ', \Sigma1-\omega^j=' num2str(sum1mwrnpi_r1,'%4.2f') ],...
        'color',colors(4,:),'fontsize',16);
    axis([0 15 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %title('Response to inflation shock')
    %print( fig_ir_s1, 'impulse-response_r11.eps', '-depsc');
    %print( fig_ir_s1, 'impulse-response_r11_png.png', '-dpng');
    
    
    fig_ir2 = figure;
    colors = get(fig_ir2,'defaultaxescolororder');
    xplot = plot((0:H-1)',xtr1(:,[pii svi vi]) ,'linewidth',2);
    hold on
    plot((0:H-1)',zeros(H,1),'-k');
    text(5,xtr1(5+1,pii)-0.005,lbls(pii),'color',colors(1,:),'fontsize',16);
    text(6.3,xtr1(6+1,vi),lbls(vi),'color',colors(3,:),'fontsize',16);
    text(5,xtr1(5+1,svi)+0.1,[char(lbls(svi)) ', \Sigma=' num2str(sums_r1,'%4.2f')],'color',colors(2,:),'fontsize',16);
    axis([0 15 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %title('Response to inflation shock')
    %print( fig_ir2, 'impulse-response_r12.eps', '-depsc');
    %print( fig_ir2, 'impulse-response_r12_png.png', '-dpng');
    
    
    fig_ir3 = figure;
    xplot = plot((0:H-1)',xtr1(:,[pii rni yi y10i]),'linewidth',2);
    hold on
    plot(ones(1,4),xtr1(2,[pii rni yi y10i]),'v');
    plot((0:H-1)',zeros(H,1),'-k');
    text(5,xtr1(5+1,pii),lbls(pii),'color',colors(1,:),'fontsize',16,'backgroundcolor','w');
    text(5,xtr1(5+1,rni),lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(1.5,xtr1(1+1,rni)-0.05,lbls(rni),'color',colors(2,:),'fontsize',16,'backgroundcolor','w');
    text(6,xtr1(6+1,yi)+0.01,lbls(yi),'color',colors(3,:),'fontsize',16);
    text(7,xtr1(7+1,y10i)-0.01,lbls(y10i),'color',colors(4,:),'fontsize',16);
    %title('Response to inflation shock')
    axis([0 10 -inf inf]);
    xlabel('Years')
    ylabel('Percent')
    %print(fig_ir3, 'impulse-response_r13.eps', '-depsc');
    %print(fig_ir3, 'impulse-response_r13_png.png', '-dpng');
    
end

% plot of inflation and growth residuals

figure
hold on;
plot((1948:2018)',[100*err(:,gi)],'-b','linewidth',2)
plot((1948:2018)',[100*err(:,pii)],'-vr','linewidth',2)
plot((1948:2018)',[0*err(:,pii)],'color','k')
axis([1945 2020 -inf inf])
ylabel('Percent')
legend('Growth','Inflation')
x0=10;
y0=10;
width=800;%550;
height=400;
set(gcf,'position',[x0,y0,width,height])
print -depsc2 grandpi.eps;
disp('growth and inflation shocks')
fprintf(' %10i %10.2f %10.2f \n',[(1948:2018)' 100*err(:,gi) 100*err(:,pii)]')


end; % ends a big if only sample 1947-2018

% standard error plot
if Ntrial > 0;
    stderr = figure;
    
    subplot(3,2,1);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,pii); xt_75(end:-1:1,pii)],'y')
    hold on
    plot((0:H-1)',xt_50(:,pii),'--r','linewidth',3);
    plot((0:H-1)',xt(:,pii),'linewidth',3);
    axis([0 10 -inf inf]);
    title('Inflation \pi');
    
    subplot(3,2,3);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,svi); xt_75(end:-1:1,svi)],'y')
    hold on
    plot((0:H-1)',xt_50(:,svi),'--r','linewidth',3);
    plot((0:H-1)',xt(:,svi),'linewidth',3);
    axis([0 10 -inf inf]);
    title('Surplus s');
    
    subplot(3,2,5);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,rni); xt_75(end:-1:1,rni)],'y')
    hold on
    plot((0:H-1)',xt_50(:,rni),'--r','linewidth',3);
    plot((0:H-1)',xt(:,rni),'linewidth',3);
    axis([0 10 -inf inf]);
    title('Bond return r^n');
    
    subplot(3,2,6);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,end); xt_75(end:-1:1,end)],'y')
    hold on
    plot((0:H-1)',xt_50(:,end),'--r','linewidth',3);
    plot((0:H-1)',xt(:,rni)-xt(:,pii),'linewidth',3);
    axis([0 10 -inf inf]);
    title('Real return r');
    
    
    subplot(3,2,4);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,vi); xt_75(end:-1:1,vi)],'y')
    hold on
    plot((0:H-1)',xt_50(:,vi),'--r','linewidth',3);
    plot((0:H-1)',xt(:,vi),'linewidth',2);
    axis([0 10 -inf inf]);
    title('Value v');
    
    subplot(3,2,2);
    fill([(0:H-1)';(H-1:-1:0)'], [xt_25(:,gi); xt_75(end:-1:1,gi)],'y')
    hold on
    plot((0:H-1)',xt_50(:,gi),'--r','linewidth',3);
    plot((0:H-1)',xt(:,gi),'linewidth',2);
    axis([0 10 -inf inf]);
    title('Growth g ');
    
    if (stdate == 1947);
        print(stderr,'std_err_ir.eps','-depsc')
    elseif (stdate == 1930); 
        print(stderr,'std_err_ir_full.eps','-depsc')
    elseif (stdate == 1980); 
        print(stderr,'std_err_ir_80.eps','-depsc')
    end
end



%%%%%%%%%%%%
% Decomposition of value v is moved to value_decomposition
%%%%%%%%%%%%


% FUNCTION DEFINITIONS


function  dt = find_decomp(xt,pii,rni,gi,svi);

fun = @(omeg) (xt(2,rni)+sum( (omeg.^(1:size(xt,1)-2)').*xt(3:end,rni))).^2;
options = optimoptions('fmincon','Display','off');
omeg = fmincon(fun,0.7,[],[],[],[],0,1,[],options);
fval = fun(omeg);
if fval > 1e-8;
    disp('omega search was not able to nail identity, usig closest omega, and error = '); 
    disp([omeg fval^0.5]);
end

sumwpi = sum( (omeg.^(0:size(xt,1)-2)').*xt(2:end,pii));
sum1wpi  = sum( (omeg.^(1:size(xt,1)-2)').*xt(3:end,pii));
sum1wrnpi = sum( (omeg.^(1:size(xt,1)-2)').*(xt(3:end,rni)-xt(3:end,pii)));
sum1mwrnpi = sum( (1-omeg.^(1:size(xt,1)-2)').*(xt(3:end,rni)-xt(3:end,pii)));

infl = xt(2,pii);
rn = xt(2,rni);
sums = sum(xt(2:end,svi));
sumg = sum(xt(2:end,gi));
sumrpi = sum(xt(3:end,rni)-xt(3:end,pii));
dt = [sumwpi sum1wpi sum1wrnpi sum1mwrnpi infl rn sums sumg sumrpi omeg];

end

function [sumwpi,sum1wpi,sum1wrnpi,sum1mwrnpi,infl,rn,sums,sumg,sumrpi,omeg] = decode(dt);
sumwpi = dt(1);
sum1wpi = dt(2);
sum1wrnpi = dt(3);
sum1mwrnpi = dt(4);
infl = dt(5);
rn = dt(6);
sums = dt(7);
sumg = dt(8);
sumrpi = dt(9);
omeg = dt(10);
end



  